Enum CodecError
pub enum CodecError {
BufferTooSmall,
Other(Box<dyn Error + Send + Sync>),
}Available on crate features
io and ancillary only.Expand description
Error that can occur when encoding or decoding ancillary data.
Variants
BufferTooSmall
The provided buffer is too small to hold the encoded data.
Other(Box<dyn Error + Send + Sync>)
Another error occurred during encoding or decoding.
Implementations
impl CodecError
impl CodecError
pub fn other(error: impl Into<Box<dyn Error + Send + Sync>>) -> CodecError
pub fn other(error: impl Into<Box<dyn Error + Send + Sync>>) -> CodecError
Create a new CodecError::Other from any error type.
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: Error + 'static,
Attempt to downcast the error to a concrete type.
Returns Some(&T) if the error is of type T, otherwise None.
pub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
pub fn downcast_mut<T>(&mut self) -> Option<&mut T>where
T: Error + 'static,
Attempt to downcast the error to a concrete type.
Returns Some(&mut T) if the error is of type T, otherwise None.